import dynclass.ClassDefinition;
import dynclass.MethodDefinition;
import junit.framework.TestCase;

/**
 * Ensure behavior of equals and hashCode methods on
 * types that might live in collections.
 */
public class CompareTests extends TestCase {

    public CompareTests(String name) {

        super(name);
    }

    private void checkEqualsAndHashcode(Object lhs, Object rhs) {

        assertTrue("This test needs two distinct but equivalent objects",
                   lhs != rhs);

        assertEquals("Test objects must be equal according to equals()", 
                     lhs, rhs);

        assertTrue("Equality not reflexive", lhs.equals(lhs));
        assertTrue("equals(null) == true", !lhs.equals(null));
        assertTrue(!lhs.equals(this));
        assertEquals("Equality not symmetric", rhs, lhs);
        assertEquals("Hashcodes differ for equal objects",
                     lhs.hashCode(), rhs.hashCode());
    }

    public void testEqualsAndHashcode() {

        MethodDefinition desc1 = new MethodDefinition("m1",
                                                      String.class,
                                                      new Class[] { Float.class });
        MethodDefinition desc2 = new MethodDefinition("m1",
                                                      String.class,
                                                      new Class[] { Float.class });

        checkEqualsAndHashcode(desc1, desc2);

        checkEqualsAndHashcode(desc1.getDescription(), desc2.getDescription());
        
        checkEqualsAndHashcode(new ClassDefinition(Object.class,
                                                   new Class[] { Cloneable.class },
                                                   new MethodDefinition[] { desc1 }),
                               new ClassDefinition(Object.class,
                                                   new Class[] { Cloneable.class },
                                                   new MethodDefinition[] { desc2 }));
    }
}
