package dynclass;

import junit.framework.TestCase;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;

public class ConstantPoolTest extends TestCase {

    public ConstantPoolTest(String name) {

        super(name);
    }

    /**
     * Make sure duplicates return the same slot number.
     */
    public void testRepeatability() {

        ConstantPool cp = new ConstantPool();
        short utf1 = cp.utf8Const("hello world");
        assertEquals(2, cp.getCount());
        assertEquals(utf1, cp.utf8Const("hello world"));

        short fr1 = cp.fieldRefConst("java/lang/String", "g", "F");
        assertEquals(fr1, cp.fieldRefConst("java/lang/String", "g", "F"));

        assertTrue(utf1 != fr1);

        assertEquals(8, cp.getCount());

        short d1 = cp.doubleConst(Math.PI);
        assertEquals(d1, cp.doubleConst(Math.PI));

        assertEquals(10, cp.getCount());
    }

    /**
     * Check output bytes.
     */
    public void testBytes() throws Exception {

        ConstantPool cp = new ConstantPool();
        cp.intConst(12);
        cp.floatConst(30);

        int floatBits = Float.floatToIntBits(30);

        long longVal = 2 * (1 + (long)Integer.MAX_VALUE);
        cp.longConst(longVal);

        cp.methodRefConst("hi", "123", "45");
        cp.methodRefConst("jk", "123", "45");
        cp.utf8Const("ABCDE");
        //cp.classConst("ABCDE");

        ByteArrayOutputStream baos = new ByteArrayOutputStream();

        cp.writePool(new DataOutputStream(baos));

        byte[] cpBytes = baos.toByteArray();
        // really bytes, I don't feel like casting
        int[] expected = {
            3, // int constant
            0,
            0,
            0,
            12,
            4, // float constant
            floatBits >> 24,
            (floatBits >> 16) % 256,
            (floatBits >> 8) % 256,
            (floatBits % 256),
            5, // long constant - cp #3,4
            0,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            1, // utf8 constant "hi" - cp #5
            0, // length of "hi"
            2, 
            104, //'h'
            105, //'i'
            7, // classRef - cp #6
            0,
            5,
            1, // utf8 "123" - cp #7
            0, // length of "123"
            3, 
            49,
            50,
            51,
            1, // utf8 "45" - cp #8
            0,
            2,
            52,
            53,
            12, // nameAndType constant - cp #9
            0,
            7,
            0,
            8,
            10, // methodRef - cp #10
            0,
            6,
            0,
            9,
            1, // utf8 "jk" - cp #11
            0,
            2,
            106,
            107,
            7, // classRef - cp #12
            0,
            11,
            10, // methodRef - cp #13
            0,
            12,
            0,
            9,
            1, // utf8 - cp #14 "ABCDE",
            0,
            5,
            65,
            66,
            67,
            68,
            69,
            //7, // classRef - cp #14
            //0,
            //12,
        };
        assertEquals(expected.length, cpBytes.length);
        for (int i=0; i < expected.length; i++) {
            assertEquals("byte: "+i, (byte) expected[i], cpBytes[i]);
        }
        assertEquals(15, cp.getCount());
    }
}
